# The main file required by programs that use r7. The small DSL is implemented here, as well at the few predefined 
# "macros" needed for File IO, & string manipulation.

require 'fileutils'

# File Modes
READ_MODE = 'r'
WRITE_MODE = 'w'
APPEND_MODE = 'a'

# Backslash Notation
NEW_LINE = "\n"
SPACE = "\s"
TAB = "\t"

# Misc
SHEBANG_PATH = '#!/usr/bin/env'
SCRIPT_NAME = $0

# This is needed to get the line number(start at 1), from the index(starts at 0).
def index_offset(index)
	index - 1
end

# Returns the file extention for the "filename".
def GET_EXTNAME(filename)
	File.extname(filename)
end

# Returns the basename of the path, and the "filename".
def GET_BASENAME(filename)
	File.basename(filename, GET_EXTNAME(filename))
end

# Opens a "file" on "READ_MODE" by default.
def ACCESS(file: '', mode: READ_MODE)
	File.open(file, mode) { |f| yield(f) }
end

# Returns a string with all of the files contents.
def READ(file: '')
	ACCESS(file: file) do |f|
		f.seek(0, :END)
		file_size = f.tell
		f.rewind
		f.read(file_size)
	end
end

# Writes "contents" to "file".
def WRITE(file: '', mode: WRITE_MODE, contents: '')
	ACCESS(file: file, mode: mode) { |f| f.write(contents) }
end

# Writes "contents" to the end of "file".
def APPEND(file: '', mode: APPEND_MODE, contents: '')
	WRITE(file: file, mode: mode, contents: contents)
end

# Returns a new string with content add in place.
# @example
#    # This is in a file named: foobar.rb
#    $foobar = true
#    # This is in an file named: replace_foobar.r7
#    # We'll make "foobar" into an instance variable instead...
#    PLACE(this_content: '@foobar = true', on_this_line: 10, in_this_file: 'foobar.rb')
#    # Results
#    @foobar = true
def PLACE(this_content: '', on_this_line: 1, in_this_file: '')
	offset = index_offset(on_this_line)
	contents = READ(file: in_this_file)
	contents = contents.split(NEW_LINE)
	contents.each_index { |i| contents[index_offset(i)] = this_content if i == on_this_line }
	# This corrects an error that occurs when there is no string that follows the index intended to be modified.
	contents[index_offset(on_this_line)] = this_content unless contents[index_offset(on_this_line)] == this_content
	contents.join(NEW_LINE)
end

# Returns a new string with content replacing a substring.
# @example
#    # This is in a file named: foobar.rb
#    class FooBar
#    end
#    # This is in an file named: replace_foobar.r7
#    # Here we are turning the class into a method definition!
#    REPLACE(this: 'class', with_this: 'def', on_this_line: 1, in_this_file: 'foobar.rb')
#    # Results
#    def FooBar
#    end
def REPLACE(this: '', with_this: '', on_this_line: 1, in_this_file: '')
	offset = index_offset(on_this_line)
	contents = READ(file: in_this_file)
	contents = contents.split(NEW_LINE)[offset].split
	contents.each_index { |i| contents[i] = with_this if contents[i].include?(this) }
	contents = contents.join(SPACE)
	PLACE(this_content: contents, on_this_line: on_this_line, in_this_file: in_this_file)
end
